import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.awt.geom.*;
import javax.swing.event.*;

/**
 * <b>Programm:</b> ComplexCalc<br>
 * <b>Copyright:</b> 2002 Andreas Gohr, Frank Schubert<br>
 * <b>License:</b> GPL2 or higher<br>
 * <br>
 * <b>Info:</b> paints the graphical representation of two complex numbers and an
 * operation on them
 */
public class Diagram extends JPanel{
  ComplexNumber cn1;
  ComplexNumber cn2;
  ComplexNumber cnr;
  String op;
  String equ = "";

  /**
   * Constructor
   */
  public Diagram(MouseListener ml){
    addMouseListener(ml);
    cn1 = new ComplexNumber();
    cn2 = new ComplexNumber();
    cnr = new ComplexNumber();
    op = "+";
  }

  /**
   * set operator and recalculate
   */
  public void setOp(String xop){
    op = xop;
    recalc();
  }

  /**
   * calculate the result value
   */
  public void recalc(){
    if(op == "+"){
      cnr = cn1.add(cn2);
    }else if (op == "-"){
      cnr = cn1.sub(cn2);
    }else if (op == "*"){
      cnr = cn1.mul(cn2);
    }else if (op == "/"){
      cnr = cn1.div(cn2);
    }
    equ = cn1.toString()+" "+op+" "+cn2.toString()+" = "+cnr.toString();
    //System.err.println(equ);
    repaint();
  }

  /**
   * Set first operand and recalculate
   */
  public void setVal1(ComplexNumber xcn){
    cn1 = xcn;
    recalc();
  }

  /**
   * Set second operand and recalculate
   */
  public void setVal2(ComplexNumber xcn){
    cn2 = xcn;
    recalc();
  }

  /**
   * Set new result value and repaint
   */
  public void setValR(ComplexNumber xcn){
    cnr = xcn;
    repaint();
  }
  
  /**
   * Calculates realworld coordinate from screen coordinate (X-Axis)
   */
  public double i2eX(double intern){
    double extern = intern - (getWidth()/2);
           extern = extern/10;
    //System.err.println("i2eX: intern "+intern+" extern "+extern);
    return extern;
  }
  
  /**
   * Calculates screen coordinate from realworld coordinate (X-Axis)
   */
  public double e2iX(double extern){
    double intern = extern*10;
           intern = intern + (getWidth()/2);
    //System.err.println("e2iX: extern "+extern+" intern "+intern);
    return intern;
  }

  /**
   * Calculates realworld coordinate from screen coordinate (Y-Axis)
   */
  public double i2eY(double intern){
    double extern = intern + (getHeight()/2);
           extern = getHeight()-extern;
           extern = extern/10;
    //System.err.println("i2eY: intern "+intern+" extern "+extern);
    return extern;
  }
  
  /**
   * Calculates screen coordinate from realworld coordinate (X-Axis)
   */
  public double e2iY(double extern){
    double intern = extern*10;
           intern = getHeight()-intern;
	   intern = intern - (getHeight()/2);
    //System.err.println("e2iY: extern "+extern+" intern "+intern);
    return intern;
  }

  /**
   * paints the diagram
   */
  public void paint(Graphics g) {
    Graphics2D g2 = (Graphics2D) g;

    Dimension d = getSize();

    //set up background
    g2.setRenderingHint( RenderingHints.KEY_ANTIALIASING,
                         RenderingHints.VALUE_ANTIALIAS_ON);
    g2.setBackground(Color.black);
    g2.clearRect(0, 0, d.width, d.height);

    //draw coordinates
    g2.setColor(Color.blue);
    g2.draw( new Line2D.Double(getWidth()/2,0,getWidth()/2,getHeight()) );
    g2.draw( new Line2D.Double(0,getHeight()/2,getWidth(),getHeight()/2) );

    //draw vectors from complex numbers:
    g2.setColor(Color.yellow);
    g2.draw( new Line2D.Double(e2iX(0),e2iY(0),e2iX(cn1.getRe()),e2iY(cn1.getIm())) );
    g2.setColor(Color.green);
    g2.draw( new Line2D.Double(e2iX(0),e2iY(0),e2iX(cn2.getRe()),e2iY(cn2.getIm())) );
    g2.setColor(Color.red);
    g2.draw( new Line2D.Double(e2iX(0),e2iY(0),e2iX(cnr.getRe()),e2iY(cnr.getIm())) );
    g2.drawString(equ,20,20);
  }
}
